/* $Id: pingv6.c,v 1.7 2001/09/09 17:08:08 jared Exp $ */

#include "config.h"

#ifdef HAVE_IPv6

/* structure that carries all the ipv6 ping related data */
struct pingv6data {
	unsigned int ident;			/* duh */
        unsigned char outpack[128];		/* the packet we output */
	unsigned char cmsgbuf[4096];		/* options */
	unsigned int cmsglen;			/* length of cmsgbuf */
	unsigned int ntransmitted;		/* duh */
	unsigned int nreceived;
	unsigned char *packet;
	unsigned char *datap;
        int counter;
	struct sockaddr_in6 ping_target;	/* what to ping */
	struct sockaddr_in6 *to;
        struct timeval lastsentat;
	char rcvd_tbl[8192];
} tempv6;

#define A(bit)          tempv6.rcvd_tbl[(bit)>>3]  /* identify byte in array */
#define B(bit)          (1 << ((bit) & 0x07))   /* identify bit in byte */
#define SET(bit)        (A(bit) |= B(bit))
#define CLR(bit)        (A(bit) &= (~B(bit)))
#define TST(bit)        (A(bit) & B(bit))


/*
 * pinger_v6 --
 *      Compose and transmit an ICMP ECHO REQUEST packet.  The IP packet
 * will be added on by the kernel.  The ID field is our UNIX process ID,
 * and the sequence number is an ascending integer.  The first 8 bytes
 * of the data portion are used to hold a UNIX "timeval" struct in VAX
 * byte-order, to compute the round-trip time.
 */
void pinger_v6(struct pingv6data *localdata, struct monitorent *here)
{
        struct icmp6hdr *icmph;
	unsigned int mx_dup_ck = 65536;
        register int cc;
	int datalen;
	int confirm = 0;
        int ret1;
	int serrno;


        icmph = (struct icmp6hdr *)localdata->outpack;
        icmph->icmp6_type = ICMPV6_ECHO_REQUEST;
        icmph->icmp6_code = 0;
        icmph->icmp6_cksum = 0;
        icmph->icmp6_sequence = localdata->ntransmitted++;
        icmph->icmp6_identifier = localdata->ident;

        CLR(icmph->icmp6_sequence % mx_dup_ck);

        cc = ICMP_PHDR_LEN + 8;                       /* skips ICMP portion */

	ret1 = sendto(glob_icmpv6_fd, (char *)localdata->outpack, cc, confirm,
		(struct sockaddr *)&localdata->ping_target, 
		sizeof(struct sockaddr_in6));

        serrno = errno;

        if (ret1 < 0 || ret1 != cc)
        {
                switch(serrno)
                {
                        case ENETUNREACH:
                                here->retval = SYSM_NETUNRCH;
                                return;
                        case EHOSTDOWN:
                        case EHOSTUNREACH:
                                here->retval =  SYSM_HOSTDOWN;
                                return;
                        default:
                        /* A new one to me */
                                perror("pingv6.c:pinger_v6:sendto");
                }
        }
        /* Track it */
        gettimeofday(&localdata->lastsentat, NULL);


}

void setup_icmpv6_fd()
{
	int retval;
	unsigned int hold = ICMP_HOLD_QUEUE;

	glob_icmpv6_fd = socket(AF_INET6, SOCK_RAW, IPPROTO_ICMPV6);
        retval = -1;
        while (retval == -1)
        {
                retval = setsockopt(glob_icmpv6_fd, SOL_SOCKET,
                        SO_RCVBUF, (char *)&hold, sizeof(hold));

                if (retval == -1)
                {
                        if (errno == ENOBUFS)
                        {
                                hold = hold- (hold / 4);
                                continue;
                        }
                        perror("pingv6.c:setsockopt");
                        print_err(1,
                                "pingv6.c:setsockopt returned -1, errno = %d",
                                errno);
                        break;
                }
        }
        print_err(0, "sysmond: INFO: hold queue set to %d for icmpv6 packets", hold);

}


void start_test_pingv6(struct monitorent *checkme)
{
	struct pingv6data *localstruct = NULL;

        if (glob_icmpv6_fd == -1)
        {
                /* If there is no icmp fd, say it's ok */
                checkme->retval = SYSM_OK;
                return;
        }
        checkme->monitordata=MALLOC(sizeof(struct pingv6data),"icmpv6-localstruct");

        memset(checkme->monitordata, 0, sizeof(struct pingv6data));

        localstruct = checkme->monitordata;

        gettimeofday(&checkme->lastserv, NULL);

        localstruct->nreceived = 0; /* zero it out */

        localstruct->ntransmitted = 0; /* zero it out */

        localstruct->datap = &localstruct->outpack[8 + sizeof(struct timeval)];

        /* initalize the variable */
        memset(&localstruct->ping_target, 0, sizeof(struct sockaddr));

        if (debug)
        {
                print_err(0, "setting up ping of host %s",
                        checkme->checkent->hostname);
        }

        /* set to */
        localstruct->to = (struct sockaddr_in6 *)&localstruct->ping_target;

        /* set family as IPv6 */
        localstruct->to->sin6_family = AF_INET6;

        localstruct->to->sin6_addr.s_addr = checkme->checkent->inaddr;

        /* do a dns lookup on the hostname we're being passed */
        localstruct->hp = gethostbyname2(checkme->checkent->hostname, AF_INET6);

        if (!localstruct->hp)  /* did we get an error doing a dns query */
        {
                /* if so, do the return thang */
                checkme->retval = SYSM_NODNS;
                FREE(localstruct);
                checkme->monitordata = NULL;
                return; /* can't forget this else we core */
        }

        /* set the family type */
        localstruct->to->sin_family = localstruct->hp->h_addrtype;

        memcpy((caddr_t)&localstruct->to->sin6_addr, localstruct->hp->h_addr,
                localstruct->hp->h_length);

        if (!(localstruct->packet = (u_char *)MALLOC(ICMP_PACKET_SIZE, "pingv6.c:packet_data")))
        {
                /* aie!! */
                print_err(0, "pingv6.c: out of memory.");
                checkme->retval = checkme->checkent->lastcheck;
                FREE(localstruct);
                checkme->monitordata = NULL;
                return;
        }

        for (localstruct->counter = 8; localstruct->counter < 128;
                ++localstruct->counter)  /* do something.. */
        {
                        *localstruct->datap++ = localstruct->counter;
                                        /* and keep doing it */
        }

        /* generate our identity for the packet */
        localstruct->ident = generate_ident();

        if (debug)
        {
                print_err(0, "pingv6.c:Created ICMP identity id of %d", localstruct->ident);
        }

        /*  Send a ping */
        pinger_v6(localstruct, checkme);

        if (debug)
        {
                print_err(0, "pingv6.c:Sent an ICMP echo-request to %s",
                        checkme->checkent->hostname);
        }

        /* Track the last time that we sent a ping */
        gettimeofday(&localstruct->lastsentat, NULL);

        /* end of setup, we send the packet, and leave it at that  --
                service_test_ping watches for replies*/

        return;

	return;
}

void service_test_pingv6(struct monitorent *checkme)
{
	return;
}

void stop_test_pingv6(struct monitorent *checkme)
{
	return;
}



#endif /* HAVE_IPv6 */

